searchData={"items":[{"type":"module","title":"EctoTemp","doc":"`EctoTemp` is `use`'d to set up a module for managing temp tables. Once set up, macros such as\n`EctoTemp.Macros.deftemptable/3`, `EctoTemp.Macros.column/3`, and macros in `EctoTemp.Factory`\nmay be used.","ref":"EctoTemp.html"},{"type":"module","title":"Examples - EctoTemp","doc":"```elixir\ndefmodule MyTest do\n  use MyProject.DataCase\n  use EctoTemp, repo: MyProject.Repo\n\n  require EctoTemp.Factory\n  alias EctoTemp.Factory\n\n  deftemptable :things_temp do\n    column :data, :string, null: false\n    column :data_with_default, :string, default: \"default value\"\n    deftimestamps()\n  end\n\n  deftemptable :other_things_temp, primary_key: false do\n    column :id, :uuid, null: false\n    column :data, :string, null: false\n    column :data_with_default, :string, default: \"default value\"\n    deftimestamps()\n  end\n\n  setup do\n    create_temp_tables()\n    :ok\n  end\n\n  test \"insert records\" do\n    Factory.insert(:things_temp, data: \"stuff\")\n  end\nend\n```","ref":"EctoTemp.html#module-examples"},{"type":"macro","title":"EctoTemp.__using__/1","doc":"Imports EctoTemp into a module.","ref":"EctoTemp.html#__using__/1"},{"type":"macro","title":"Options - EctoTemp.__using__/1","doc":"* `:repo` - required - the module defining your Ecto.Repo callbacks.","ref":"EctoTemp.html#__using__/1-options"},{"type":"macro","title":"Example - EctoTemp.__using__/1","doc":"```elixir\ndefmodule MyModule do\n  use EctoTemp, repo: MyProject.Repo\nend\n```","ref":"EctoTemp.html#__using__/1-example"},{"type":"module","title":"EctoTemp.Factory","doc":"","ref":"EctoTemp.Factory.html"},{"type":"macro","title":"EctoTemp.Factory.insert/1","doc":"Inserts values into a temporary table.","ref":"EctoTemp.Factory.html#insert/1"},{"type":"macro","title":"Params: - EctoTemp.Factory.insert/1","doc":"* struct (optional) - a struct defined the schema used by the data migration.\n  * table_name\n  * attrs (optional) - a keyword list of attributes to insert","ref":"EctoTemp.Factory.html#insert/1-params"},{"type":"macro","title":"Notes: - EctoTemp.Factory.insert/1","doc":"* If not given a struct, and the temporary table has a primary key, then we return the `id` of the inserted row.\n  * If given a struct, and the temporary table has a primary key, then we do a `Repo.get` using the `id` of the\n    inserted row, and return the result as a struct.\n  * If the temporary table has no primary key, then we return the list of values returned by postgres. This list\n    is probably ordered by the order in which the columns are defined on the temp table???","ref":"EctoTemp.Factory.html#insert/1-notes"},{"type":"macro","title":"Examples - EctoTemp.Factory.insert/1","doc":"import EctoTemp.Factory\n\n    insert(:thing_with_no_primary_key) == []\n    insert(:thing_with_no_primary_key, some_thing: \"hi\") == [\"hi\"]\n    insert(:thing_with_primary_key) == 1\n    insert(:thing_with_primary_key, some_thing: \"hi\") == 2\n    %MyDataMigration.Cycle{id: 1} = insert(MyDataMigration.Cycle, :cycles, started_at: ~N[2020-02-03 00:00:00])","ref":"EctoTemp.Factory.html#insert/1-examples"},{"type":"macro","title":"EctoTemp.Factory.insert/2","doc":"","ref":"EctoTemp.Factory.html#insert/2"},{"type":"macro","title":"EctoTemp.Factory.insert/3","doc":"","ref":"EctoTemp.Factory.html#insert/3"},{"type":"function","title":"EctoTemp.Factory.uuid/0","doc":"Generates a UUID that may be inserted directly into a `:uuid` field. Use this in preference of\n`Ecto.UUID.generate/0` or `Ecto.UUID.bingenerate/0`, as those will not dump their string values\nto the binary format expected by postgres.","ref":"EctoTemp.Factory.html#uuid/0"},{"type":"module","title":"EctoTemp.Macros","doc":"","ref":"EctoTemp.Macros.html"},{"type":"macro","title":"EctoTemp.Macros.column/3","doc":"Add a column to a table definition.\n\nThis must be called within a `deftemptable` block, or a CompileError will be raised.","ref":"EctoTemp.Macros.html#column/3"},{"type":"macro","title":"Opts - EctoTemp.Macros.column/3","doc":"| name | type | default | description |\n| `default` | term()  |      | Provides a default value when none is provided. |\n| `null`    | boolean | true | Determines whether null values are allowed in a column. |","ref":"EctoTemp.Macros.html#column/3-opts"},{"type":"macro","title":"EctoTemp.Macros.create_temp_tables/0","doc":"Runs through previously defined `@ecto_temporary_tables` to insert temporary tables.\nThis should be called in a setup block, which needs to be defined **after** all\n`deftemptable` definitions.","ref":"EctoTemp.Macros.html#create_temp_tables/0"},{"type":"macro","title":"EctoTemp.Macros.deftemptable/3","doc":"Creates a temporary table that will be rolled back at the end of the\ncurrent test transaction. If the table name is given as `:things`, then\nthe actual temporary table will be created as `things`. As this may\noverlap with existing tables, it is recommended to use a `temp` prefix\nof suffix when defining temp tables.","ref":"EctoTemp.Macros.html#deftemptable/3"},{"type":"macro","title":"Examples - EctoTemp.Macros.deftemptable/3","doc":"deftemptable :person_temp do\n      column :thing_id, :integer, null: false\n      deftimestamps()\n    end\n\n    deftemptable :thing_temp, primary_key: false do\n      column :description, :string, null: false\n      column :comment, :string\n    end","ref":"EctoTemp.Macros.html#deftemptable/3-examples"},{"type":"macro","title":"Opts - EctoTemp.Macros.deftemptable/3","doc":"| name | type | default | description |\n| `primary_key` | boolean | true | When true, adds an `:id` field of type `:bigserial` |","ref":"EctoTemp.Macros.html#deftemptable/3-opts"},{"type":"macro","title":"EctoTemp.Macros.deftimestamps/0","doc":"Adds `inserted_at` and `updated_at` to a table definition.\n\nThis must be called within a `deftemptable` block, or a CompileError will be raised.","ref":"EctoTemp.Macros.html#deftimestamps/0"},{"type":"extras","title":"EctoTemp","doc":"# EctoTemp\n\nEctoTemp provides macros and helper functions for creating and utilizing temporary tables in\nPostgres. Among other use cases, this library can be utilized to test data migrations (where the\nunderying table may change over time) or ecto type extensions in isolation, where no one persistent\ntable exist.\n\nThe temporary tables created by this library are transaction local, using `ON COMMIT DROP`. The\n[PostgreSQL documentation](https://www.postgresql.org/docs/current/sql-createtable.html) on creating\ntables describes this setting as follows:\n\n> The temporary table will be dropped at the end of the current transaction block. When used on a\n> partitioned table, this action drops its partitions and when used on tables with inheritance\n> children, it drops the dependent children.\n\nFor use in `ExUnit`, this means that any temporary tables created in the context of the\n`Ecto.Adapters.SQL.Sandbox` are scoped to the test's database connection, and are deleted at the end\nof each test.","ref":"overview.html"},{"type":"extras","title":"Usage - EctoTemp","doc":"After adding `:ecto_temp` to a project's Mix dependencies, a test can `use EctoTemp, repo:\n `. Temporary tables are defined with `EctoTemp.Macros.deftemptable/3` and\n`EctoTemp.Macros.column/2`.\n\nTemp tables are actually created in the database with `EctoTemp.Macros.create_temp_tables/0`.\n\n```elixir\ndefmodule MyTest do\n  use Test.DataCase, async: true\n  use EctoTemp, repo: MyProject.Repo\n\n  deftemptable :things_temp do\n    column :data, :string, null: false\n    column :data_with_default, :string, default: \"default value\"\n    deftimestamps()\n  end\n\n  setup do\n    create_temp_tables()\n    :ok\n  end\nend\n```\n\nIn tests, `EctoTemp.Factory.insert/3` can be used to insert data into a temporary table. As a macro,\nit must be used via `import` or `require`.\n\n```elixir\ndefmodule MyTest do\n  # ...\n\n  require EctoTemp.Factory\n\n  test \"inserts data into a temp table\" do\n    Factory.insert(:things_temp, data: \"stuff\")\n  end\nend\n```\n\nAn example of a real project using `EctoTemp` to test an `Ecto.Type` extensions can be found in the\n[ecto_date_time_range](https://github.com/synchronal/ecto_date_time_range/blob/91b52a634799431ee16f5cdc524850198561866a/test/ecto/utc_date_time_range_test.exs)\nproject.","ref":"overview.html#usage"},{"type":"extras","title":"Data Migrations","doc":"# Data Migrations\n\nDatabase data migrations face at least two primary problems with regard to testing: database schemas\nwill change in the future; application modules will change in the future. Tests that rely on either\nface a high likelihood of failing.\n\n`Ecto` provides a mechanism for overriding the database table used by a query, via `Ecto.put_meta/2`\nwith the `:source` attribute, or via `{table, Schema}` when making queries.","ref":"data_migrations.html"},{"type":"extras","title":"Example Data Migration - Data Migrations","doc":"Imagine a data migration that takes a bunch of things, modifies one or more fields, and creates\nother related things.\n\nThis data migration can define its own schema modules, representing the schema at the time of\nmigration. At any place a database interaction occurs, the table name for each entity must be\nspecified. To ensure that this is the case, the schemas define their tables as\n`replace_me_at_runtime`, so that code that forgets to override it will blow up.\n\n```elixir\ndefmodule Core.DataMigrations.MyMigration do\n  import Ecto.Query\n\n  defmodule Thing do\n    use Ecto.Schema\n    import Ecto.Changeset\n\n    schema \"replace_me_at_runtime\" do\n      field :data, :string\n      timestamps()\n    end\n\n    @doc \"\"\"\n    Note that when constructing a query, `{table_name, Thing}` is used\n    in place of `Thing`. This overrides the database table used by the\n    query.\n    \"\"\"\n    def all(table_name),\n      do: from(things in {table_name, Thing})\n\n    @doc \"\"\"\n    When modifying data, `Ecto.put_meta/2` is utilized to enfore the\n    table name.\n    \"\"\"\n    def update_changeset(thing, table_name) do\n      thing\n      |> Ecto.put_meta(source: table_name)\n      |> change(data: thing.data <> \"-ish\")\n    end\n  end\n\n  defmodule OtherThing do\n    use Ecto.Schema\n    import Ecto.Changeset\n\n    schema \"replace_me_at_runtime\" do\n      field :thing_id, :bigint\n      timestamps()\n    end\n\n    def insert_changeset(thing, table_name) do\n      %OtherThing{}\n      |> Ecto.put_meta(source: table_name)\n      |> change(thing_id: thing.id)\n    end\n  end\n\n  @doc \"\"\"\n  Table names are injected by the tests, but default to the real tables used by\n  the application.\n  \"\"\"\n  def run(things_table \\\\ \"things\", other_things_table \\\\ \"other_things\") do\n    Core.Repo.transaction(fn ->\n      Thing.all(things_table)\n      |> Repo.stream()\n      |> Stream.map(&Thing.update_changeset(&1, things_table))\n      |> Stream.map(&update_thing/1)\n      |> Stream.map(&OtherThing.insert_changset(&1, other_things_table))\n      |> Stream.map(&insert_other_thing/1)\n      |> Stream.run()\n    end, timeout: 60_000)\n  end\n\n  def update_thing(thing_changeset),\n    do: thing_changeset |> Core.Repo.update!()\n\n  def insert_other_thing(other_thing_changeset),\n    do: other_thing_changeset |> Core.Repo.insert!()\nend\n```","ref":"data_migrations.html#example-data-migration"},{"type":"extras","title":"Example Test - Data Migrations","doc":"The test can rely on the schemas defined by the migration, and inject temporary tables at runtime:\n\n```elixir\ndefmodule Core.DataMigrations.MyMigrationTest do\n  use Test.DataCase, async: true\n  use EctoTemp, repo: Core.Repo\n\n  alias Core.DataMigrations.MyMigration, as: Migration\n  require EctoTemp.Factory\n\n  deftemptable :temp_things do\n    column :data, :string, null: false\n    deftimestamps()\n  end\n\n  deftemptable :temp_other_things do\n    column :thing_id, :integer, null: false\n    deftimestamps()\n  end\n\n  setup do\n    create_temp_tables()\n    :ok\n  end\n\n  describe \"run\" do\n    test \"does stuff\" do\n      Factory.insert(:temp_things, data: \"stuff\")\n      assert {\"temp_other_things\", Migration.OtherThing} |> Core.Repo.all() == []\n\n      # inject tables\n      Migration.run(\"temp_things\", \"temp_other_things\")\n\n      [thing] = {\"temp_things\", Migration.Thing} |> Core.Repo.all()\n      assert thing.data == \"stuff-ish\"\n\n      assert [other_thing] =\n          {\"temp_other_things\", Migration.OtherThing} |> Core.Repo.all()\n\n      assert other_thing.thing_id == thing.id\n    end\n  end\nend\n```","ref":"data_migrations.html#example-test"},{"type":"extras","title":"Change Log","doc":"# Change Log","ref":"changelog.html"},{"type":"extras","title":"Unreleased - Change Log","doc":"","ref":"changelog.html#unreleased"},{"type":"extras","title":"2.0.0 - Change Log","doc":"- Disambiguate Factory macro variations into separate heads.\n- Test against Elixir 1.18.\n- **Breaking change:** Drop support for Elixir 1.15.","ref":"changelog.html#2-0-0"},{"type":"extras","title":"1.2.0 - Change Log","doc":"- require Elixir 1.15 or greater\n- test against the latest version of Erlang and Elixir","ref":"changelog.html#1-2-0"},{"type":"extras","title":"1.1.0 - Change Log","doc":"- Drop support for Elixir 1.13 and older.","ref":"changelog.html#1-1-0"},{"type":"extras","title":"1.0.1 - Change Log","doc":"- Update deps.","ref":"changelog.html#1-0-1"},{"type":"extras","title":"1.0.0 - Change Log","doc":"- Add `EctoTemp.Factory.uuid/0`.\n- Do not automatically append `_temp` to table names.","ref":"changelog.html#1-0-0"},{"type":"extras","title":"0.1.3 - Change Log","doc":"- Add `:primary_key` info to documentation.","ref":"changelog.html#0-1-3"},{"type":"extras","title":"0.1.2 - Change Log","doc":"- Add overview, data migrations to documentation.","ref":"changelog.html#0-1-2"},{"type":"extras","title":"0.1.1 - Change Log","doc":"- `:ecto` and `:ecto_sql` are required in any environment that uses `:ecto_temp`.\n- Use Apache 2.0 license.\n- Updated project URL.","ref":"changelog.html#0-1-1"},{"type":"extras","title":"0.1.0 - Change Log","doc":"- Initial release.","ref":"changelog.html#0-1-0"},{"type":"extras","title":"License","doc":"# License\n\nCopyright 2022 synchronal.dev\n\nLicensed under the Apache License, Version 2.0 (the \"License\"); you may not use this file except in\ncompliance with the License. You may obtain a copy of the License at\n\n http://www.apache.org/licenses/LICENSE-2.0\n\nUnless required by applicable law or agreed to in writing, software distributed under the License is\ndistributed on an \"AS IS\" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\nimplied. See the License for the specific language governing permissions and limitations under the\nLicense.","ref":"license.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.36.1"}}